unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, IBCustomDataSet, IBQuery, IBDatabase, Spin,
  ZAbstractRODataset, ZDataset, ZConnection;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    HostName: TEdit;
    btnGo: TButton;
    Memo1: TMemo;
    Label2: TLabel;
    Label3: TLabel;
    btnStop: TButton;
    Threshold: TSpinEdit;
    ZConnection1: TZConnection;
    ZReadOnlyQuery1: TZReadOnlyQuery;
    Label5: TLabel;
    Protocol: TComboBox;
    procedure btnGoClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FStop: Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses DBLogDlg;

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  Protocol.ItemIndex := 0;
end;

procedure TForm1.btnGoClick(Sender: TObject);
const
  colour: array [0..2] of string = ('Red', 'Blu', 'Yel');

var
  hits: array [0..59, 0..6] of Integer;  // Count of hits received on each pack in each hit location, over all games.
  packs: array [0..59] of string;

  procedure DoAcacia(p: Integer);
  const
    locations: array [0..6] of string = ('chest', 'back', 'left shoulder', 'right shoulder', 'laser', '?', '?');
  var
    pack: Integer;
    i: Integer;
    game: Integer;
    h: array [0..6] of Integer;  // Count of hits received on a single pack in a single game
    sum: Integer;
  begin
    pack := p * 3 mod 60 + p div 20;  // pack = 0, 3, 6, 9, ... 57, 1, 4, 7, 10, ... 58, 2, 5, 8, 11, ... 59.

    // Query just for hits on this pack.  I do this as 60 small queries instead of one big one because the Interbase transaction was returning 'Out of memory'.
    ZReadOnlyQuery1.SQL.Clear;
    ZReadOnlyQuery1.SQL.Add('SELECT H.Game_ID, ' +
      'H.Location_1, H.Location_2, H.Location_3, H.Location_4, H.Location_5, G.Start_Time ' +
      'FROM T2Hits H ' +
      'JOIN T2Games G ON H.Game_ID = G.Game_ID ' +
      'WHERE H.HitOn_ID = ' + IntToStr(pack) +
      ' ORDER BY H.Game_ID');
    ZReadOnlyQuery1.Open;

    while not ZReadOnlyQuery1.Eof and not FStop do begin
      game := ZReadOnlyQuery1.FieldByName('Game_ID').AsInteger;
      for i := 1 to 5 do
        h[i] := 0;

      while (not ZReadOnlyQuery1.Eof) and (game = ZReadOnlyQuery1.FieldByName('Game_ID').AsInteger)  do begin
        for i := 1 to 5 do
          if pack < 60 then begin
            Inc(h[i], ZReadOnlyQuery1.FieldByName('Location_' + IntToStr(i)).AsInteger);
            Inc(hits[pack][i], ZReadOnlyQuery1.FieldByName('Location_' + IntToStr(i)).AsInteger);
          end;

        Application.ProcessMessages;
        ZReadOnlyQuery1.Next;
      end;  // while game = Game_ID

      sum := h[1] + h[2] + h[3] + h[4] + h[5];
      if (pack <= 60) and (sum > Threshold.Value) then
        for i := 1 to 5 do
          if h[i] = 0 then
            Memo1.Lines.Add(Format('%s %2d: game %3d at '+
              DateTimeToStr(ZReadOnlyQuery1.FieldByName('Start_Time').AsDateTime) +
              ': %d hits on the whole pack, but no hits on the %s.',
              [colour[pack mod 3], pack div 3 + 1, game, sum, locations[i]]));
    end;  // while !Eof
    Memo1.Lines.Add('');
    ZReadOnlyQuery1.Close;
  end;

  procedure DoNexus(pack: Integer);
  const
    locations: array [0..6] of string = ('weapon', 'front', 'left front shoulder', 'right front shoulder', 'left back shoulder', 'right back shoulder', 'back');
  var
    i: Integer;
    game: Integer;
    h: array [0..6] of Integer;  // Count of hits received on a single pack in a single game
    eventtype: Integer;
    sum: Integer;
  begin
    ZReadOnlyQuery1.SQL.Clear;
    ZReadOnlyQuery1.SQL.Add('SELECT E.Game_ID, Player_ID, Event_Type, G.Start_Time, P.Description ' +
      'FROM ng_player_event_log E ' +
      'JOIN ng_game_stats G ON E.Game_ID = G.Game_ID ' +
      'JOIN ng_packs P ON E.Player_ID = P.Pack_ID ' +
      'WHERE Player_ID = ' + IntToStr(pack) +
      ' ORDER BY E.Game_ID');

    ZReadOnlyQuery1.Open;

    packs[pack] := ZReadOnlyQuery1.FieldByName('Description').AsString;

    while not ZReadOnlyQuery1.Eof and not FStop do begin
      game := ZReadOnlyQuery1.FieldByName('Game_ID').AsInteger;
      for i := 0 to 6 do
        h[i] := 0;

      while (not ZReadOnlyQuery1.Eof) and (game = ZReadOnlyQuery1.FieldByName('Game_ID').AsInteger)  do begin
        eventtype := ZReadOnlyQuery1.FieldByName('Event_Type').AsInteger;
        if eventtype in [14..27] then begin
          Inc(h[eventtype mod 7]);  // Hits on location 14 or 21 increment h[0]; hits on 15 or 22 increment h[1], etc.
          Inc(hits[pack][eventtype mod 7]);
        end;

        Application.ProcessMessages;
        ZReadOnlyQuery1.Next;
      end;  // while game = Game_ID

      sum := h[0] + h[1] + h[2] + h[3] + h[4] + h[5] + h[6];
      if sum > Threshold.Value then
        for i := 0 to 6 do
          if (i in [0,1,2,3,6]) and (h[i] = 0) then
            Memo1.Lines.Add(Format('%s: game %3d at %s: %d hits on the whole pack, but no hits on the %s.',
              [ZReadOnlyQuery1.FieldByName('Description').AsString, game,
               DateTimeToStr(ZReadOnlyQuery1.FieldByName('Start_Time').AsDateTime), sum, locations[i]]));
    end;  // while !Eof
    Memo1.Lines.Add('');
    ZReadOnlyQuery1.Close;
  end;

var
  i: Integer;
  isMySql: Boolean;
  pack: Integer;
  s: string;
  totalhits: Integer;
begin
  Screen.Cursor := crHourGlass;
  FStop := False;
  btnGo.Enabled := False;
  try
    for pack := 0 to 59 do
      for i := 0 to 6 do
        hits[pack][i] := 0;

    isMySql := Pos('mysql', Protocol.Text) > 0;

    ZConnection1.HostName := HostName.Text;
    ZConnection1.Protocol := Protocol.Text;
    if isMySql then begin
      ZConnection1.Database := 'ng_system';
      ZConnection1.User := 'root';
      ZConnection1.Password := 'password';
    end else begin
      ZConnection1.Database := 'C:\Program Files\P&CMicros\Database\ZoneSystems.GDB';
      ZConnection1.User := 'sysdba';
      ZConnection1.Password := 'masterkey';
    end;
    ZConnection1.Connect;
    try
      for pack := 0 to 59 do
        if isMySql then
          DoNexus(pack)
        else
          DoAcacia(pack);
    finally
      Memo1.Lines.Add('');
      if isMySql then
        Memo1.Lines.Add(' Pack       Weapon   Front   LeftF  RightF   LeftR  RightR    Back   Total')
      else
        Memo1.Lines.Add(' Pack                Chest    Back    Left   Right   Laser           Total');
      Memo1.Lines.Add(  '------      ------   -----   -----  ------   -----  ------    ----   -----');

      for pack := 0 to 59 do begin
        totalhits := hits[pack][0] + hits[pack][1] + hits[pack][2] + hits[pack][3] + hits[pack][4] + hits[pack][5] + hits[pack][6];
        if totalhits > 0 then
        begin
          if isMySql then begin
            s := packs[pack] + ':';
            while Length(s) < 10 do
              s := s + ' ';
          end else
            s := Format('%s %2d:   ', [colour[pack mod 3], pack div 3 + 1]);
          for i := 0 to 6 do
            s := s + Format('%8d', [hits[pack][i]]);
          Memo1.Lines.Add(s);
        end;
      end;

      Memo1.Lines.Add('');
      for pack := 0 to 59 do begin
        totalhits := hits[pack][0] + hits[pack][1] + hits[pack][2] + hits[pack][3] + hits[pack][4] + hits[pack][5] + hits[pack][6];
        if totalhits > 0 then
        begin
          if isMySql then begin
            s := packs[pack] + ':';
            while Length(s) < 10 do
              s := s + ' ';
          end else
            s := Format('%s %2d:   ', [colour[pack mod 3], pack div 3 + 1]);

          for i := 0 to 6 do
            s := s + Format(' %6.1f%%', [hits[pack][i] * 100.0 / totalhits]);

          s := s + Format('%8d', [totalhits]);
          Memo1.Lines.Add(s);
        end;
      end;
    end;
  finally
    Screen.Cursor := crDefault;
    btnGo.Enabled := True;
    ZReadOnlyQuery1.Close;
    ZConnection1.Disconnect;
  end;
end;

procedure TForm1.btnStopClick(Sender: TObject);
begin
  FStop := True;
end;

end.
